package ext.data.proxy {

[Native("Ext.data.proxy.LocalStorage", require)]
/**
 * The LocalStorageProxy uses the new HTML5 localStorage API to save →<code>ext.data.Model</code>
 * data locally on the client browser. HTML5 localStorage is a key-value store (e.g. cannot save
 * complex objects like JSON), so LocalStorageProxy automatically serializes and deserializes data
 * when saving and retrieving it.
 * <p>localStorage is extremely useful for saving user-specific information without needing to build
 * server-side infrastructure to support it. Let's imagine we're writing a Twitter search
 * application and want to save the user's searches locally so they can easily perform a saved
 * search again later. We'd start by creating a Search model:</p>
 * <pre>
 * Ext.define('Search', {
 *     fields: ['id', 'query'],
 *     extend: 'Ext.data.Model',
 *     proxy: {
 *         type: 'localstorage',
 *         id: 'twitter-Searches'
 *     }
 * });
 * </pre>
 * <p>Our Search model contains just two fields - id and query - plus a Proxy definition. The only
 * configuration we need to pass to the LocalStorage proxy is an →<code>id</code>. This is important
 * as it separates the Model data in this Proxy from all others. The localStorage API puts all data
 * into a single shared namespace, so by setting an id we enable LocalStorageProxy to manage
 * the saved Search data.</p>
 * <p>Saving our data into localStorage is easy and would usually be done with a
 * →<code>ext.data.Store</code>:</p>
 * <pre>
 * // our Store automatically picks up the LocalStorageProxy defined on the Search model
 * var store = Ext.create('Ext.data.Store', {
 *     model: "Search"
 * });
 *
 * // loads any existing Search data from localStorage
 * store.load();
 *
 * // now add some Searches
 * store.add({ query: 'Sencha Touch' });
 * store.add({ query: 'Ext JS' });
 *
 * // finally, save our Search data to localStorage
 * store.sync();
 * </pre>
 * <p>The LocalStorageProxy automatically gives our new Searches an id when we call store.sync().
 * It encodes the Model data and places it into localStorage. We can also save directly
 * to localStorage, bypassing the Store altogether:</p>
 * <pre>
 * var search = Ext.create('Search', { query: 'Sencha Animator' });
 *
 * // uses the configured LocalStorageProxy to save the new Search to localStorage
 * search.save();
 * </pre>
 * <p><b>Limitations</b></p>
 * <p>If this proxy is used in a browser where local storage is not supported, the constructor
 * will throw an error. A local storage proxy requires a unique ID which is used as a key in which
 * all record data are stored in the local storage object.</p>
 * <p>It's important to supply this unique ID as it cannot be reliably determined otherwise. If no id
 * is provided but the attached store has a storeId, the storeId will be used. If neither option
 * is presented the proxy will throw an error.</p>
 * @see ext.data.Model
 * @see #id
 * @see ext.data.Store
 * @see https://docs.sencha.com/extjs/7.2.0/classic/Ext.data.proxy.LocalStorage.html Original Ext JS documentation of 'Ext.data.proxy.LocalStorage'
 */
public class LocalStorageProxy extends ext.data.proxy.WebStorageProxy {
  /**
   * @param config @inheritDoc
   */
  public function LocalStorageProxy(config:ext.data.proxy.LocalStorageProxy = null) {
    super();
  }
}
}